import { Server } from 'socket.io'
import { TSocket } from '#utilities/types'
import prisma from '#utilities/prisma'
import characterRepository from '#repositories/characterRepository'
import { ItemType, ItemRarity } from '@prisma/client'
import { gameMasterLogger } from '#utilities/logger'

type Payload = {
  id: string
  name: string
  description: string | null
  itemType: ItemType
  stackable: boolean
  rarity: ItemRarity
  spriteId: string | null
}

export default class ItemUpdateEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:item:update', this.handleObjectUpdate.bind(this))
  }

  private async handleObjectUpdate(data: Payload, callback: (success: boolean) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId as number)
    if (!character) return callback(false)

    if (character.role !== 'gm') {
      return callback(false)
    }

    try {
      await prisma.item.update({
        where: { id: data.id },
        data: {
          name: data.name,
          description: data.description,
          itemType: data.itemType,
          stackable: data.stackable,
          rarity: data.rarity,
          spriteId: data.spriteId
        }
      })

      return callback(true)
    } catch (error) {
      gameMasterLogger.error(`Error updating item: ${error instanceof Error ? error.message : String(error)}`)
      return callback(false)
    }
  }
}
